#include "SevSeg.h"
SevSeg sevseg;  //Initiate a seven segment controller object

//buttons for start/stop and reset
int button1 = 14;
int button2 = 15;

//track current number of milliseconds to add to currentTime
unsigned long currentMillis = 0;
//track millis to find currentMillis next cycle
unsigned long previousMillis = 0;
//track current time, reset when clock rolls back from 59:59
unsigned long currentTime = 0;
//centiseconds used for display, reduced to 0-59
unsigned long centiTime = 0;
//seconds used for display, reduced to 0-59
unsigned long secTime = 0;
//final 4-digit time displayed on 7-seg
unsigned long timeDisplay = 0;

//internval to adjust for inaccurate timekeeping
const int interval = 31495;
int intervalCount = 0;

//track pauseMode
bool pauseMode = true;

void setup() {
  //this section is used for setting up 7-seg displays
  byte numDigits = 4;
  byte digitPins[] = { 10, 11, 12, 13 };
  byte segmentPins[] = { 2, 3, 4, 5, 6, 7, 8, 9 };
  bool resistorsOnSegments = 0;
  // variable above indicates that 4 resistors were placed on the digit pins.
  // set variable to 1 if you want to use 8 resistors on the segment pins.
  sevseg.begin(COMMON_CATHODE, numDigits, digitPins, segmentPins, resistorsOnSegments);
  sevseg.setBrightness(90);
}

void loop() {
  //check current millis count
  currentMillis = millis();

  //if not paused, add difference current and previous millis to current time count
  if (!pauseMode) currentTime = currentTime + (currentMillis - previousMillis);

  //assign new millis value to previousMillis
  previousMillis = millis();
  //increase intervalCount by 1
  intervalCount++;

  //if interval has been reached, add 100ms
  if (intervalCount >= interval) {
    intervalCount = 0;
    //only adjust for interval if not paused
    if(!pauseMode) currentTime = currentTime + 100;
  }

  //if start/stop button pressed
  if (digitalRead(button1)) {
    //toggle pauseMode
    if (pauseMode) pauseMode = false;
    else pauseMode = true;
    //delay to avoid double-pressing button
    delay(300);
  }

  //if reset button pressed, set current time to 0
  if (digitalRead(button2)) currentTime = 0;

  //reset current time every minute
  if (currentTime >= 60000) currentTime = currentTime - 60000;

  //calculate centiTime to 0-99
  centiTime = currentTime / 10;
  while (centiTime > 99) centiTime = centiTime - 100;

  //calculate secTime and reduce to 0-59
  secTime = currentTime / 1000;
  while (secTime > 59) secTime = secTime - 60;

  //combine second and centisecond times into 4-digit format
  timeDisplay = 100 * secTime + centiTime;
  //set and refresh 7-seg displays
  sevseg.setNumber(timeDisplay, 2);
  sevseg.refreshDisplay();  // Must run repeatedly
}
